/*
 *  tslib/src/ts_config.c
 *
 *  Copyright (C) 2001 Russell King.
 *
 * This file is placed under the LGPL.  Please see the file
 * COPYING for more details.
 *
 * $Id: ts_config.c,v 1.5 2004/10/19 22:01:27 dlowder Exp $
 *
 * Read the configuration and load the appropriate drivers.
 */
#include "config.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <termios.h>
#include <ctype.h>
#include <unistd.h>
#include <errno.h>
#include "tslib-private.h"

/* Maximum line size is BUF_SIZE - 2 
 * -1 for fgets and -1 to test end of line
 */
#define BUF_SIZE 512
#define BAUD_RATE  19200

static void setline(int fd, int flags, int speed)
{
	struct termios t;
#ifdef DEBUG	
	printf("set line.\n");
#endif

	tcgetattr(fd, &t);

	t.c_cflag = flags | CREAD | HUPCL | CLOCAL;
	t.c_iflag = IGNBRK | IGNPAR;
	t.c_oflag = 0;
	t.c_lflag = 0;
	t.c_cc[VMIN ] = 1;
	t.c_cc[VTIME] = 0;

	cfsetispeed(&t, speed);
	cfsetospeed(&t, speed);

	tcsetattr(fd, TCSANOW, &t);
}


static int SerialSetBaud(int fd, struct termios* pTIOS, int nBaud)
{
#ifdef DEBUG
	printf("Setting baud rate to %d\n",nBaud);
#endif
	
	/* conver baud rate to tios macro */
	int baudRate = B19200;
	switch (nBaud)
	{
		case 38400: baudRate = B38400; break;
		case 19200: baudRate = B19200; break;
		case 9600: baudRate = B9600; break;
		case 4800: baudRate = B4800; break; /* for testing, maybe */
		case 2400: baudRate = B2400; break; /* for testing, maybe */
		case 1200: baudRate = B1200; break; /* for testing, maybe */
	}

	/* change baud rate */
	cfsetispeed(pTIOS, baudRate);
	cfsetospeed(pTIOS, baudRate);
	//if (tcsetattr (fd, TCSANOW, pTIOS))
	//	return 1;

	return 0;
}

static int SerialSetSampRate(int fd)
{
	unsigned char cmd ; /* = 0xff  Dummy data */;
	
	//stop the sample
	cmd = '0';
	if (write(fd, &cmd, 1) != 1)
		return -1;
	
	usleep(20000);
	
	//start the sample
	cmd = '1'; 
	if (write(fd, &cmd, 1) != 1)
		return -1;	

	return 0;
}

static int SerialConfigTTY(int fd, int nBaudRate)
{
	struct termios tios;

#ifdef DEBUG	
	printf("config tty.\n");	
#endif

	/* configure tty */
	if (isatty(fd))
	{
		/* set up default port parameters */
		/*
		if (tcgetattr (fd, &tios))
		{
			fprintf(stderr,"Failed to get port params: %s\n", strerror(errno));
			return -1;
		}
		*/ 
		
		/*
		tios.c_iflag &= ~(IGNBRK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON);
		tios.c_oflag &= ~OPOST;
		tios.c_lflag &= ~(ECHO|ECHONL|ICANON|ISIG|IEXTEN);
		tios.c_cflag &= ~(CSIZE|PARENB);
		tios.c_cflag |= CS8|CLOCAL;
		tios.c_cflag &= ~(CSTOPB); /* 1 stop bit */
		//tios.c_cflag &= ~(CSIZE); /* 8 data bits */
		//tios.c_cflag |= CS8;
   	    //tios.c_cflag &= ~(PARENB); /* no parity */
		//tios.c_iflag |= IXOFF;		/* flow control XOff */
		//tios.c_cc[VMIN] = 1;		/* vmin value */
		//tios.c_cc[VTIME] = 0;		/* vtime value */
		
		setline(fd, CS8, B19200);
		/*
		if (SerialSetBaud(fd, &tios, nBaudRate))
		{
			fprintf(stderr,"Failed to set BaudRate: %s\n", strerror(errno));
			return -1;
		}
		
		if (tcsetattr (fd, TCSANOW, &tios))
		{
			fprintf(stderr,"Failed to set port params: %s\n", strerror(errno));
			return -1;
		}
		*/
	}
	else /* not tty */
	{
		return -1;
	}
	
	if ( SerialSetSampRate(fd))
	{
		fprintf(stderr,"Failed to set sampRate: %s\n", strerror(errno));
		return -1;
	}

	return 0;
}


int ts_config(struct tsdev *ts)
{
	char buf[BUF_SIZE], *p;
	FILE *f;
	int line = 0;
	int ret = 0;

	char *conffile;
	
	/*
	if( SerialConfigTTY(ts->fd, BAUD_RATE))
	{
		fprintf(stderr, "config serial port failed: %s\n", strerror(errno));
		return -1;
	}
	*/

	if( (conffile = getenv("TSLIB_CONFFILE")) == NULL) {
		conffile = strdup (TS_CONF);
	}

	f = fopen(conffile, "r");
	if (!f) {
		perror("Couldnt open tslib config file");
		return -1;
	}

	buf[BUF_SIZE - 2] = '\0';
	while ((p = fgets(buf, BUF_SIZE, f)) != NULL) {
		char *e;
		char *tok;
		char *module_name;

		line++;

		/* Chomp */
		e = strchr(p, '\n');
		if (e) {
			*e = '\0';
		}

		/* Did we read a whole line? */
		if (buf[BUF_SIZE - 2] != '\0') {
			ts_error("%s: line %d too long\n", conffile, line);
			break;
		}

		tok = strsep(&p, " \t");
		
		/* Ignore comments or blank lines.
		 * Note: strsep modifies p (see man strsep)
		 */

		if (p==NULL || *tok == '#')
			continue;

		/* Search for the option. */
		if (strcasecmp(tok, "module") == 0) {
			module_name = strsep(&p, " \t");
			ret = ts_load_module(ts, module_name, p);
		}
		else if (strcasecmp(tok, "module_raw") == 0) {
			module_name = strsep(&p, " \t");
			ret = ts_load_module_raw(ts, module_name, p);
		} else {
			ts_error("%s: Unrecognised option %s:%d:%s\n", conffile, line, tok);
			break;
		}
		if (ret != 0) {
			ts_error("Couldnt load module %s, %s\n", module_name, strerror(errno));
			break;
		}
	}

	if (ts->list_raw == NULL) {
		ts_error("No raw modules loaded.\n");
		ret = -1;
	}

	fclose(f);

	return ret;
}



